import pytest
import trio_asyncio

import triopg


async def execute_queries(triopg_conn, asyncpg_conn):
    @trio_asyncio.aio_as_trio
    async def _asyncpg_query(sql):
        return await asyncpg_conn.execute(sql)

    # Execute without transaction
    await triopg_conn.execute(
        """
        DROP TABLE IF EXISTS users;
        CREATE TABLE IF NOT EXISTS users (
            _id SERIAL PRIMARY KEY,
            user_id VARCHAR(32) UNIQUE
        )"""
    )
    assert await _asyncpg_query("""SELECT * FROM users""") == "SELECT 0"

    # Execute in transaction without exception
    async with triopg_conn.transaction():
        await triopg_conn.execute("INSERT INTO users (user_id) VALUES (1)")
    assert await _asyncpg_query("""SELECT * FROM users""") == "SELECT 1"

    # Execute in transaction raising exception, request shound not be executed
    with pytest.raises(Exception):
        async with triopg_conn.transaction():
            await triopg_conn.execute("INSERT INTO users (user_id) VALUES (2)")
            raise Exception()

    assert await _asyncpg_query("""SELECT * FROM users""") == "SELECT 1"


@pytest.mark.trio
async def test_triopg_connection(
        asyncio_loop, asyncpg_conn, postgresql_connection_specs
):
    conn = await triopg.connect(**postgresql_connection_specs)
    await execute_queries(conn, asyncpg_conn)
    await conn.close()

    with pytest.raises(triopg.InterfaceError):
        await conn.execute("SELECT * FROM users")


@pytest.mark.trio
async def test_triopg_pool(
        asyncio_loop, asyncpg_conn, postgresql_connection_specs
):
    pool = await triopg.create_pool(**postgresql_connection_specs)
    try:
        async with pool.acquire() as conn:
            await execute_queries(conn, asyncpg_conn)
    finally:
        await pool.close()

    with pytest.raises(triopg.InterfaceError):
        async with pool.acquire() as conn:
            pass


@pytest.mark.trio
async def test_triopg_pool_context_manager(
        asyncio_loop, asyncpg_conn, postgresql_connection_specs
):
    async with triopg.create_pool_cm(**postgresql_connection_specs) as pool:
        async with pool.acquire() as conn:
            await execute_queries(conn, asyncpg_conn)

    with pytest.raises(triopg.InterfaceError):
        async with pool.acquire() as conn:
            pass



"""Top-level package for triopg."""

from ._version import __version__
from ._triopg import connect, create_pool, create_pool_cm
from .exceptions import *  # NOQA

__all__ = (
    '__version__', 'connect', 'create_pool', 'create_pool_cm'
) + exceptions.__all__  # NOQA



import pytest
import trio_asyncio
import asyncio


@trio_asyncio.trio2aio
async def work_in_asyncio():
    await asyncio.sleep(0)


@pytest.fixture()
async def asyncio_loop():
    async with trio_asyncio.open_loop() as loop:
        yield loop


@pytest.fixture()
async def asyncio_fiture_with_fixtured_loop(asyncio_loop):
    await work_in_asyncio()
    yield 42


@pytest.fixture()
async def asyncio_fixture_own_loop():
    async with trio_asyncio.open_loop():
        await work_in_asyncio()
        yield 42


@pytest.mark.trio
async def test_no_fixture():
    async with trio_asyncio.open_loop():
        await work_in_asyncio()


@pytest.mark.trio
async def test_half_fixtured_asyncpg_conn(asyncio_fixture_own_loop):
    pass


@pytest.mark.trio
async def test_fixtured_asyncpg_conn(asyncio_fiture_with_fixtured_loop):
    pass



import pytest
import trio_asyncio
import asyncpg
from asyncpg.cluster import Cluster
import tempfile

import triopg


@pytest.fixture()
async def asyncio_loop():
    async with trio_asyncio.open_loop() as loop:
        yield loop


@pytest.fixture(scope='session')
def cluster():
    cluster_dir = tempfile.mkdtemp()
    cluster = Cluster(cluster_dir)

    cluster.init()
    try:
        cluster.start(port='dynamic')
        yield cluster
        cluster.stop()
    finally:
        cluster.destroy()


@pytest.fixture
def postgresql_connection_specs(cluster):
    return {'database': 'postgres', **cluster.get_connection_spec()}


@pytest.fixture()
async def asyncpg_conn(asyncio_loop, postgresql_connection_specs):
    conn = await trio_asyncio.aio_as_trio(asyncpg.connect
                                          )(**postgresql_connection_specs)
    try:
        yield conn
    finally:
        await trio_asyncio.aio_as_trio(conn.close)()


@pytest.fixture()
async def triopg_conn(asyncio_loop, postgresql_connection_specs):
    conn = await triopg.connect(**postgresql_connection_specs)
    try:
        yield conn
    finally:
        await conn.close()



from functools import wraps, partial
import trio
import asyncpg
import trio_asyncio


def _shielded(f):
    @wraps(f)
    async def wrapper(*args, **kwargs):
        with trio.open_cancel_scope(shield=True):
            return await f(*args, **kwargs)

    return wrapper


@trio_asyncio.aio_as_trio
async def connect(*args, **kwargs):
    return TrioConnectionProxy(await asyncpg.connect(*args, **kwargs))


async def create_pool(*args, **kwargs):
    pool = TrioPoolProxy(*args, **kwargs)
    await pool._async__init__()
    return pool


def create_pool_cm(*args, **kwargs):
    return TrioPoolProxy(*args, **kwargs)


class TrioTransactionProxy:
    def __init__(self, asyncpg_transaction):
        self._asyncpg_transaction = asyncpg_transaction

    @trio_asyncio.aio_as_trio
    async def __aenter__(self, *args):
        return await self._asyncpg_transaction.__aenter__(*args)

    @_shielded
    @trio_asyncio.aio_as_trio
    async def __aexit__(self, *args):
        return await self._asyncpg_transaction.__aexit__(*args)


class TrioConnectionProxy:
    def __init__(self, asyncpg_conn):
        self._asyncpg_conn = asyncpg_conn

    def transaction(self, *args, **kwargs):
        asyncpg_transaction = self._asyncpg_conn.transaction(*args, **kwargs)
        return TrioTransactionProxy(asyncpg_transaction)

    def __getattr__(self, attr):
        target = getattr(self._asyncpg_conn, attr)

        if callable(target):

            @wraps(target)
            @trio_asyncio.aio_as_trio
            async def wrapper(*args, **kwargs):
                return await target(*args, **kwargs)

            # Only generate the function wrapper once per connection instance
            setattr(self, attr, wrapper)

            return wrapper

        return target

    @_shielded
    @trio_asyncio.aio_as_trio
    async def close(self):
        return await self._asyncpg_conn.close()


class TrioPoolAcquireContextProxy:
    def __init__(self, asyncpg_acquire_context):
        self._asyncpg_acquire_context = asyncpg_acquire_context

    @trio_asyncio.aio_as_trio
    async def __aenter__(self, *args):
        proxy = await self._asyncpg_acquire_context.__aenter__(*args)
        return TrioConnectionProxy(proxy._con)

    @_shielded
    @trio_asyncio.aio_as_trio
    async def __aexit__(self, *args):
        return await self._asyncpg_acquire_context.__aexit__(*args)


class TrioPoolProxy:
    def __init__(self, *args, **kwargs):
        self._asyncpg_create_pool = partial(
            asyncpg.create_pool, *args, **kwargs
        )
        self._asyncpg_pool = None

    def acquire(self):
        return TrioPoolAcquireContextProxy(self._asyncpg_pool.acquire())

    @_shielded
    @trio_asyncio.aio_as_trio
    async def close(self):
        return await self._asyncpg_pool.close()

    def terminate(self):
        return self._asyncpg_pool.terminate()

    async def _async__init__(self):
        if not self._asyncpg_pool:
            self._asyncpg_pool = await trio_asyncio.aio_as_trio(
                self._asyncpg_create_pool
            )()
        return self._asyncpg_pool

    async def __aenter__(self):
        await self._async__init__()
        return self

    async def __aexit__(self, *exc):
        return await self.close()



from setuptools import setup, find_packages

exec(open("triopg/_version.py", encoding="utf-8").read())

LONG_DESC = open("README.rst", encoding="utf-8").read()

setup(
    name="triopg",
    version=__version__,
    description="PostgreSQL client for Trio based on asyncpg",
    url="https://github.com/python-trio/triopg",
    long_description=LONG_DESC,
    author="Emmanuel Leblond",
    author_email="emmanuel.leblond@gmail.com",
    license="MIT -or- Apache License 2.0",
    packages=find_packages(),
    install_requires=[
        "trio>=0.7.0",
        "trio-asyncio>=0.9.0",
        "asyncpg>=0.15.0",
    ],
    keywords=["async", "trio", "sql", "postgresql", "asyncpg"],
    python_requires=">=3.6",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "License :: OSI Approved :: Apache Software License",
        "Framework :: Trio",
        "Operating System :: POSIX :: Linux",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Programming Language :: SQL",
    ],
)
